function [to,so,uo,uo_v] = MSLocUknTxPosMvgObj_Conf(cj,K,M,N)
% function [to,so,uo,uo_v] = MSLocUknTxPosMvgObj_Conf(cj,K,M,N)
%
% This program is used to create randomly generated configurations. 
%
% Usage: [to,so,uo,uv] = MSLocUknTxPosMvgObj_Conf(cj,K,M,10);
% 
% Input Parameter List:
% cj:   Number of configurations
% K:    dimension
% M:    Number of transmitters
% 10:   Maximum number of receivers
% 
% Output Parameter List:
% to:   (KxM)xcj transmitter position matrix.
% so:   (KxN)xcj receiver positions matrix.
%       N: Number of receivers.
% uo:   Kxcj object position matrix.
% uv:   Kxcj object velocity matrix.
% 
% Reference:
% J. Pei, G. Wang, K. C. Ho, and L. Huang, "Reducing bias for multistatic 
% localization of a moving object by transmitter at unknown position," IEEE 
% Trans. Aerosp. Electron. Syst., vol. 59, no. 5, pp. 5324-5341, Oct. 2023.
% 
% Jian Pei, Gang Wang, K. C. Ho and Lei Huang	Nov. 1, 2023
%
%       Copyright (C) 2023
%       Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China.
%       wanggang@nbu.edu.cn
%

for m = 1:cj
    rand('state',20+m-1);   
%====creat object position====
    u_x = -3000+3000*rand(1);
    u_y = 3000*rand(1);
    u_z = 3000+1000*rand(1);
    uo(:,m) = [u_x;u_y;u_z];
%====creat transmitter position====
    for i = 1:M
        flag1 = [];
        while(1)
            sensor_tx = 4000*rand(1);
            sensor_ty = 4000*rand(1);
            sensor_tz = 500+500*rand(1);
            sensor_t0 = [sensor_tx;sensor_ty;sensor_tz];
            distance_ut = norm(uo(:,m)-sensor_t0);
            if(distance_ut > 1000)%1000
                if (i==1)
                    to((i-1)*K+1:i*K,m) = sensor_t0;
                    break;
                else
                    for ii = 1:i-1
                        flag1(ii) = (norm(sensor_t0-to((ii-1)*K+1:ii*K,m)) >= 1000);
                    end
                    if flag1 == ones(1,ii)
                        to((i-1)*K+1:i*K,m) = sensor_t0;
                        break;
                    end
                end
            end
        end
    end
%====creat sensor positions====    
    for j = 1:N
        flag2 = [];flag3 = [];
        while(1)
            sensor_sx = -4000+4000*rand(1);
            sensor_sy = -4000+8000*rand(1);
            sensor_sz = 500+500*rand(1);
            sensor_s0 = [sensor_sx;sensor_sy;sensor_sz];
            distance_us = norm(uo(:,m)-sensor_s0);
            if(distance_us > 1000)
                for i = 1:M
                    flag2(i) = (norm(sensor_s0-to((i-1)*K+1:i*K,m)) >= 700);
                end
                if (flag2 == ones(1,M))
                    if (j == 1)
                            so(:,j,m) = sensor_s0;
                        break;
                    else
                        for jj = 1:j-1
                            flag3(jj) = (norm(sensor_s0-so(:,jj,m)) >= 1500);
                        end
                        if flag3 == ones(1,jj)
                            so(:,j,m) = sensor_s0;
                            break;
                        end
                    end
                end
            end
        end
    end
%====creat object velocity====  
    rand('state',300-m);
    uo_v(:,m) = 20*rand(K,1);
end
